
//---------------------------------------------------------------------------

// This software is Copyright (c) 2011 Embarcadero Technologies, Inc. 
// You may only use this software if you are an authorized licensee
// of Delphi, C++Builder or RAD Studio (Embarcadero Products).
// This software is considered a Redistributable as defined under
// the software license agreement that comes with the Embarcadero Products
// and is subject to that software license agreement.

//---------------------------------------------------------------------------
unit main;

interface

uses
  System.SysUtils, System.Variants, System.Classes, System.UITypes, System.UIConsts, FMX.Types, FMX.Forms, FMX.Controls,
  FMX.Filter, FMX.Layouts, FMX.ListBox, FMX.Objects, FMX.Colors, FMX.Ani, FMX.Grid,
  FMX.Types3D, FMX.Layers3D, Data.Bind.EngExt, Fmx.Bind.DBEngExt, System.Rtti,
  System.Bindings.Outputs, Fmx.Bind.Editors, Data.Bind.Components, FMX.Dialogs,
  FMX.Edit;

type
  TfrmMain = class(TForm)
    LayoutControl: TLayout;
    Splitter1: TSplitter;
    PanelControl: TPanel;
    btnLoad: TButton;
    PanelOrigin: TPanel;
    TrackBarSharpen: TTrackBar;
    lbSharpenDesc: TLabel;
    lbSharpenValue: TLabel;
    Label2: TLabel;
    OpenDialogPicture: TOpenDialog;
    LayoutPictures: TLayout;
    PanelImageWork: TPanel;
    ImageDest: TImage;
    SplitterHoriz: TSplitter;
    PanelControlImages: TPanel;
    SplitterVert: TSplitter;
    ImageContrast: TImage;
    GroupBoxOriginSource: TGroupBox;
    ImageOrigin: TImage;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    TrackBarBrightness: TTrackBar;
    TrackBarContrast: TTrackBar;
    TrackBarPenSize: TTrackBar;
    lbBrightnessDesc: TLabel;
    lbContrastDesc: TLabel;
    lbPenSizeDesc: TLabel;
    lbBrightnessValue: TLabel;
    lbContrastValue: TLabel;
    lbPenSizeValue: TLabel;
    PanelContrastImage: TPanel;
    lbContrastImageHeader: TLabel;
    PanelVectorizePreparedImage: TPanel;
    ImageVectorizePrepared: TImage;
    lbVectorizePreparedImageHeader: TLabel;
    PanelMonochromeImage: TPanel;
    ImageMonochrome: TImage;
    lbMonochromeImageHeader: TLabel;
    PanelScaleControl: TPanel;
    Label1: TLabel;
    TrackBarScale: TTrackBar;
    lbScaleDesc: TLabel;
    lbScaleValue: TLabel;
    txfWidth: TEdit;
    txfHeight: TEdit;
    lbWidth: TLabel;
    lbHeight: TLabel;
    PanelSharpeImage: TPanel;
    ImageSharpe: TImage;
    lbSharpeHeader: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnLoadPictureClick(Sender: TObject);
    procedure TrackBarSharpenChange(Sender: TObject);
    procedure Splitter1Resize(Sender: TObject);
    procedure TrackBarBrightnessChange(Sender: TObject);
    procedure TrackBarContrastChange(Sender: TObject);
    procedure TrackBarPenSizeChange(Sender: TObject);
    procedure ImageContrastClick(Sender: TObject);
    procedure ImageMonochromeClick(Sender: TObject);
    procedure ImageVectorizePreparedClick(Sender: TObject);
    procedure TrackBarScaleChange(Sender: TObject);
    procedure ImageSharpeClick(Sender: TObject);
  private
    { Private declarations }
    m_sRunDir: String;
    m_iFocusedImage: Integer;
    m_lWithUpdate: Boolean;
    procedure SetTrackBarDefaultsWithoutActions;
    procedure UpdatePreview;
    procedure UpdateContrastPreview;
    procedure UpdateSharpePreview;
    procedure UpdateMonochromePreview;
    procedure UpdatePaperSketchPreview;
    procedure UpdateDest;
  public
    { Public declarations }
    FilterContrast: TFilter;
    FilterRecContrast: TFilterRec;
    FilterSharpe: TFilter;
    FilterRecSharpe: TFilterRec;
    FilterMonochrome: TFilter;
    FilterRecMonochrome: TFilterRec;
    FilterPaperSketch: TFilter;
    FilterRecPaperSketch: TFilterRec;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.fmx}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
 FilterSharpe := FilterByName('Sharpen');
 FilterRecSharpe := FilterClassByName('Sharpen').FilterAttr;
 FilterMonochrome := FilterByName('Monochrome');
 FilterRecMonochrome := FilterClassByName('Monochrome').FilterAttr;
 FilterContrast := FilterByName('Contrast');
 FilterRecContrast := FilterClassByName('Contrast').FilterAttr;
 FilterPaperSketch := FilterByName('PaperSketch');
 FilterRecPaperSketch := FilterClassByName('PaperSketch').FilterAttr;
 self.SetTrackBarDefaultsWithoutActions;
 m_lWithUpdate := True;
 ImageDest.DeleteChildren;
 GetDir(0,m_sRunDir);
 m_iFocusedImage := 0;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
 if FilterSharpe <> nil then
   FilterSharpe.Free;
 if FilterMonochrome <> nil then
   FilterMonochrome.Free;
 if FilterContrast <> nil then
   FilterContrast.Free;
 if FilterPaperSketch <> nil then
   FilterPaperSketch.Free;
end;

procedure TfrmMain.ImageContrastClick(Sender: TObject);
begin
 if m_iFocusedImage > 0 then begin
   self.m_iFocusedImage := 1;
   self.UpdateDest;
 end;
end;

procedure TfrmMain.ImageSharpeClick(Sender: TObject);
begin
 if m_iFocusedImage > 0 then begin
   self.m_iFocusedImage := 2;
   self.UpdateDest;
 end;
end;

procedure TfrmMain.ImageMonochromeClick(Sender: TObject);
begin
 if m_iFocusedImage > 0 then begin
   self.m_iFocusedImage := 3;
   self.UpdateDest;
 end;
end;

procedure TfrmMain.ImageVectorizePreparedClick(Sender: TObject);
begin
 if m_iFocusedImage > 0 then begin
   self.m_iFocusedImage := 4;
   self.UpdateDest;
 end;
end;

procedure TfrmMain.Splitter1Resize(Sender: TObject);
var
 l_oPos: TPosition;
begin
 l_oPos := Splitter1.Position;
end;

procedure TfrmMain.TrackBarBrightnessChange(Sender: TObject);
begin
 if m_lWithUpdate then begin
   FilterContrast.Values[FilterRecContrast.Values[0].Name] := TTrackBar(Sender).Value;
   lbBrightnessValue.Text := FloatToStrF(TTrackBar(Sender).Value, ffFixed, 10, 2);
   self.UpdateContrastPreview;
 end;
end;

procedure TfrmMain.TrackBarContrastChange(Sender: TObject);
begin
 if m_lWithUpdate then begin
   FilterContrast.Values[FilterRecContrast.Values[1].Name] := TTrackBar(Sender).Value;
   lbContrastValue.Text := FloatToStrF(TTrackBar(Sender).Value, ffFixed, 10, 2);
   self.UpdateContrastPreview;
 end;
end;

procedure TfrmMain.TrackBarSharpenChange(Sender: TObject);
begin
 if m_lWithUpdate then begin
   FilterSharpe.Values[FilterRecSharpe.Values[0].Name] := TTrackBar(Sender).Value;
   lbSharpenValue.Text := FloatToStrF(TTrackBar(Sender).Value, ffFixed, 10, 2);
   self.UpdateSharpePreview;
 end;
end;

procedure TfrmMain.TrackBarPenSizeChange(Sender: TObject);
begin
 if m_lWithUpdate then begin
   FilterPaperSketch.Values[FilterRecPaperSketch.Values[0].Name] := TTrackBar(Sender).Value;
   lbPenSizeValue.Text := FloatToStrF(TTrackBar(Sender).Value, ffFixed, 10, 2);
   self.UpdatePaperSketchPreview;
   self.UpdateDest;
 end;
end;

procedure TfrmMain.TrackBarScaleChange(Sender: TObject);
begin
 if m_lWithUpdate then begin
   FilterAffineTransform.Values[FilterRecAffineTransform.Values[2].Name] := TTrackBar(Sender).Value;
   lbScaleValue.Text := FloatToStrF(TTrackBar(Sender).Value, ffFixed, 10, 2);
   self.UpdateDest;
 end;
end;

procedure TfrmMain.btnLoadPictureClick(Sender: TObject);
var
 l_sFileName, l_sFileExt: String;
begin
 OpenDialogPicture.InitialDir := m_sRunDir;
 if OpenDialogPicture.Execute then begin
  l_sFileName := OpenDialogPicture.FileName;
  l_sFileExt  := UpperCase(System.SysUtils.ExtractFileExt(l_sFileName));
  if (l_sFileExt = '.BMP') or (l_sFileExt = '.JPG') or (l_sFileExt = '.PNG') then begin
    try
      ImageOrigin.Bitmap.LoadFromFile(l_sFileName);
      self.SetTrackBarDefaultsWithoutActions;
      m_iFocusedImage := 4;
      self.UpdatePreview;
    except end;
  end;
 end;
end;

procedure TfrmMain.SetTrackBarDefaultsWithoutActions;
var
 l_sDesc: String;
begin
 m_lWithUpdate := False;
 //
 l_sDesc := FilterRecSharpe.Values[0].Name + ' (' +
            FloatToStrF(FilterRecSharpe.Values[0].Min, ffFixed, 10, 2) +
            ' ... ' +
            FloatToStrF(FilterRecSharpe.Values[0].Max, ffFixed, 10, 2) + ')';
 lbSharpenDesc.Text := l_sDesc;
 TrackBarSharpen.Value  := FilterRecSharpe.Values[0].Default;
 TrackBarSharpen.Min    := FilterRecSharpe.Values[0].Min;
 TrackBarSharpen.Max    := FilterRecSharpe.Values[0].Max;
 lbSharpenValue.Text    := FloatToStrF(TrackBarSharpen.Value, ffFixed, 10, 2);
 //
 l_sDesc := FilterRecContrast.Values[0].Name + ' (' +
            FloatToStrF(FilterRecContrast.Values[0].Min, ffFixed, 10, 2) +
            ' ... ' +
            FloatToStrF(FilterRecContrast.Values[0].Max, ffFixed, 10, 2) + ')';
 lbBrightnessDesc.Text := l_sDesc;
 TrackBarBrightness.Value := FilterRecContrast.Values[0].Default;
 TrackBarBrightness.Min   := FilterRecContrast.Values[0].Min;
 TrackBarBrightness.Max   := FilterRecContrast.Values[0].Max;
 lbBrightnessValue.Text := FloatToStrF(TrackBarBrightness.Value, ffFixed, 10, 2);
 //
 l_sDesc := FilterRecContrast.Values[1].Name + ' (' +
            FloatToStrF(FilterRecContrast.Values[1].Min, ffFixed, 10, 2) +
            ' ... ' +
            FloatToStrF(FilterRecContrast.Values[1].Max, ffFixed, 10, 2) + ')';
 lbContrastDesc.Text := l_sDesc;
 TrackBarContrast.Value := FilterRecContrast.Values[1].Default;
 TrackBarContrast.Min   := FilterRecContrast.Values[1].Min;
 TrackBarContrast.Max   := FilterRecContrast.Values[1].Max;
 lbContrastValue.Text   := FloatToStrF(TrackBarContrast.Value, ffFixed, 10, 2);
 //
 l_sDesc := FilterRecPaperSketch.Values[0].Name + ' (' +
            FloatToStrF(FilterRecPaperSketch.Values[0].Min, ffFixed, 10, 2) +
            ' ... ' +
            FloatToStrF(FilterRecPaperSketch.Values[0].Max, ffFixed, 10, 2) + ')';
 lbPenSizeDesc.Text := l_sDesc;
 TrackBarPenSize.Value  := FilterRecPaperSketch.Values[0].Default;
 TrackBarPenSize.Min    := FilterRecPaperSketch.Values[0].Min;
 TrackBarPenSize.Max    := FilterRecPaperSketch.Values[0].Max;
 lbPenSizeValue.Text    := FloatToStrF(TrackBarPenSize.Value, ffFixed, 10, 2);
 //
 l_sDesc := FilterRecAffineTransform.Values[2].Name + ' (' +
            FloatToStrF(FilterRecAffineTransform.Values[2].Min, ffFixed, 10, 2) +
            ' ... ' +
            FloatToStrF(FilterRecAffineTransform.Values[2].Max, ffFixed, 10, 2) + ')';
 lbScaleDesc.Text := l_sDesc;
 TrackBarScale.Value  := FilterRecAffineTransform.Values[2].Default;
 TrackBarScale.Min    := FilterRecAffineTransform.Values[2].Min;
 TrackBarScale.Max    := FilterRecAffineTransform.Values[2].Max;
 lbScaleValue.Text    := FloatToStrF(TrackBarScale.Value, ffFixed, 10, 2);
 //
 m_lWithUpdate := True;
end;

procedure TfrmMain.UpdatePreview;
begin
 if m_lWithUpdate then begin
   self.UpdateContrastPreview;
 end;
end;

procedure TfrmMain.UpdateContrastPreview;
begin
 if m_lWithUpdate then begin
   if FilterContrast <> nil then begin
     FilterContrast.ValuesAsBitmap['Input'] := ImageOrigin.Bitmap;
     ImageContrast.Bitmap := TBitmap(FilterContrast.ValuesAsBitmap['output']);
   end;
   self.UpdateSharpePreview;
   self.UpdateMonochromePreview;
 end;
end;

procedure TfrmMain.UpdateSharpePreview;
begin
 if m_lWithUpdate then begin
   if FilterSharpe <> nil then begin
     FilterSharpe.ValuesAsBitmap['Input'] := ImageContrast.Bitmap;
     ImageSharpe.Bitmap := TBitmap(FilterSharpe.ValuesAsBitmap['output']);
   end;
   self.UpdateMonochromePreview;
 end;
end;

procedure TfrmMain.UpdateMonochromePreview;
begin
 if m_lWithUpdate then begin
   if FilterMonochrome <> nil then begin
     FilterMonochrome.ValuesAsBitmap['Input'] := ImageSharpe.Bitmap;
     ImageMonochrome.Bitmap := TBitmap(FilterMonochrome.ValuesAsBitmap['output']);
   end;
   self.UpdatePaperSketchPreview;
   self.UpdateDest;
 end;
end;

procedure TfrmMain.UpdatePaperSketchPreview;
begin
 if m_lWithUpdate then begin
   if FilterPaperSketch <> nil then begin
     FilterPaperSketch.ValuesAsBitmap['Input'] := ImageMonochrome.Bitmap;
     ImageVectorizePrepared.Bitmap := TBitmap(FilterPaperSketch.ValuesAsBitmap['output']);
   end;
 end;
end;

procedure TfrmMain.UpdateDest;
var
 l_oRect: TRectangle;
begin
 ImageDest.DeleteChildren;
 l_oRect := PanelContrastImage.FindStyleResource('panelstyle') as TRectangle;
 if Assigned(l_oRect) then
   l_oRect.Fill.Color := TAlphaColors.White;
 l_oRect := PanelSharpeImage.FindStyleResource('panelstyle') as TRectangle;
 if Assigned(l_oRect) then
   l_oRect.Fill.Color := TAlphaColors.White;
 l_oRect := PanelMonochromeImage.FindStyleResource('panelstyle') as TRectangle;
 if Assigned(l_oRect) then
   l_oRect.Fill.Color := TAlphaColors.White;
 l_oRect := PanelVectorizePreparedImage.FindStyleResource('panelstyle') as TRectangle;
 if Assigned(l_oRect) then
   l_oRect.Fill.Color := TAlphaColors.White;
 if m_iFocusedImage = 1 then begin
   ImageDest.Bitmap.Assign(ImageContrast.Bitmap);
   l_oRect := PanelContrastImage.FindStyleResource('panelstyle') as TRectangle;
   if Assigned(l_oRect) then
     l_oRect.Fill.Color := TAlphaColors.Steelblue;
 end
 else if m_iFocusedImage = 2 then begin
   ImageDest.Bitmap.Assign(ImageSharpe.Bitmap);
   l_oRect := PanelSharpeImage.FindStyleResource('panelstyle') as TRectangle;
   if Assigned(l_oRect) then
     l_oRect.Fill.Color := TAlphaColors.Steelblue;
 end
 else if m_iFocusedImage = 3 then begin
   ImageDest.Bitmap.Assign(ImageMonochrome.Bitmap);
   l_oRect := PanelMonochromeImage.FindStyleResource('panelstyle') as TRectangle;
   if Assigned(l_oRect) then
     l_oRect.Fill.Color := TAlphaColors.Steelblue;
 end
 else if m_iFocusedImage = 4 then begin
   ImageDest.Bitmap.Assign(ImageVectorizePrepared.Bitmap);
   l_oRect := PanelVectorizePreparedImage.FindStyleResource('panelstyle') as TRectangle;
   if Assigned(l_oRect) then
     l_oRect.Fill.Color := TAlphaColors.Steelblue;
 end;
end;

end.
